/*	cmsinit_simple - Initialize GPRS connection
 *
 *	Authors: joshua.randall@orange-ftgroup.com
 *               JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *               YU Yijun     <yijun.yu@orange-ftgroup.com>   
 *
 * GPL V2 license applies
 *	
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <sys/soundcard.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <time.h>

/* for select */
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

/* for shmget, shmat */
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <errno.h>

/* for stat */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

/* for signal */
#include <signal.h>

#include "serial.h"

/* Network-Specific Settings */
#define NET_MCCMNC "31026"
#define NET_APN "internet2.voicestream.com"

/* Serial Port Handle */
int PE1;

/* DTR Control Lines? */
int DTRII1;
int DTRII2;

struct termios options;
struct timespec sleeptime;

int rsize;
char buf[8193];

fd_set rfds;
struct timeval tv;
int retval;
int selectn;
int timeout_s;
int timeout_us;

struct stat statbuf;

int shmid;
int shmaddr;
char *data;

int outq;
struct sigaction alarmaction;

int state = 0;
char *laststr;
char *lastresp;

#if 0
/* SIGALRM handler */
void
handlealarm (int x)
{
  printf ("handlealarm()!\n");

  /* post power-on state */
  if (state == 1) {
    /* retry last writeread */
    printf ("retrying writeread\n");
    writeread (laststr, lastresp);
  }

  /* during power-on */
  if (state == 0) {
    /* here tapi goes into a select loop, until the alarm goes off for power-on timeout */
    ioctl (DTRII2, SNDCTL_TMR_STOP | TCSETSW, 0);

    options.c_iflag = 0;
    options.c_oflag = 0x2da98;
    /*  options.c_cflag=0x40016000;
       options.c_lflag=0x40016004; */
    options.c_cflag = shmaddr;
    options.c_lflag = shmaddr + 4;
    options.c_line = 4;
    //  options.c_cc="\x00\x00\x00\x00\x00\x00\x00\x05\x00\x00\x00\x00\x00\x00\x00\x08\x60\x01\x40";
    options.c_cc[VMIN] = 0;
    options.c_cc[VTIME] = 0;
    ioctl (DTRII2, SNDCTL_TMR_TIMEBASE | TCGETS, options);      // = 0

    data[8] = 0;

    ioctl (DTRII2, SNDCTL_TMR_START | TCSETS, 0);       // = 0

    data[8] = 1;

    //alarm(8)                                = 0
    //select(8, [4 6 7], NULL, NULL, {0, 500000}) = 1 (in [4], left {0, 500000})
    do {
      FD_ZERO (&rfds);
      FD_SET (PE1, &rfds);
      tv.tv_sec = timeout_s;
      tv.tv_usec = timeout_us;
      retval = select (selectn, &rfds, NULL, NULL, &tv);
    } while (retval == 0);

    ioctl (DTRII2, SNDCTL_TMR_SELECT | TCSETAF, 0xbffff654);    // = 0

    data[8] = 0;
    ioctl (PE1, FIONREAD, &rsize);      //                = 0
    printf ("have %d bytes to read\n", rsize);
    read (PE1, &buf, rsize);    // "\376\340\34\340\34\340\34\374\f\200\0IIII\1\2\3\4\5", 4096);// = 20
    printf ("read: [%s]\n", buf);
    write (PE1, "AT+PPMUX=0\r", 11);
    printf ("wrote: [AT+PPMUX=0]\n");
  }
}

#endif

/* signal software multiplexer (/mnt/cellon/cec/bin/multi) to turn multiplexing off (through shared memory interface) */
void
multi_off ()
{
  data[0] = 1;
}

/* signal software multiplexer (/mnt/cellon/cec/bin/multi) to turn multiplexing on (through shared memory interface) */
void
multi_on ()
{
  data[0] = 2;
}

#if 0
/* send AT command to GSM modem and wait for response */
int
writeread (char *writestr, char *resp)
{
  char mystr[8193] = "";
  int len = strlen (writestr);

  laststr = writestr;
  lastresp = resp;
  int numread = 0;
  char *bufptr = mystr;

  alarm (30);
  strcpy (mystr, writestr);
  strcat (mystr, "\r");
  write (PE1, mystr, len + 1);
  //  write(PE1, "\r", 1);
  printf ("writing [%s]...", writestr);
  do {
    ioctl (PE1, TIOCOUTQ, &outq);
    printf (".");
    usleep (100);
  } while (outq > 0);
  printf ("done.\nreading...");
  strncpy (mystr, "", 8192);
  do {                          // until found resp
    printf ("waiting on %s for %s\n", resp, writestr);
    do {                        // until select 
      FD_ZERO (&rfds);
      FD_SET (PE1, &rfds);
      tv.tv_sec = timeout_s;
      tv.tv_usec = timeout_us;
      retval = select (selectn, &rfds, NULL, NULL, &tv);
      printf (".");
    } while (retval == 0);
    ioctl (DTRII2, SNDCTL_TMR_SELECT | TCSETAF, 0xbffff654);    // = 0

    ioctl (PE1, FIONREAD, &rsize);      //[46]                = 0
    //    alarm(0);
    printf ("have %d bytes to read\n", rsize);
    numread = read (PE1, bufptr, rsize);        //"\r\n+PPURC:256,1\r\n\r\n+CIEV: 0,4\r\n\r\n"..., 4096);// = 46
    printf ("read string : %s\n", bufptr);
    printf ("done.\nread %d bytes into mystr=[%s].\n", numread, mystr);
    bufptr = mystr + strlen (mystr);    /* read next on top of null terminating char */
  } while (!strstr (mystr, resp) && !strstr (mystr, "ERROR"));
  alarm (0);
  if (strstr (mystr, "ERROR")) {
    /* received error.  give up for now */
    printf ("got error, bailing out!\n");
    return (1);
  } else {
    printf ("have %s, done with writeread.\n", resp);
    return (0);
  }
}
#endif

void
setAdvancedOptions (int fd, speed_t baud)
{
  struct termios options;
  struct termios options_cpy;

  fcntl (fd, F_SETFL, 0);

  // get the parameters
  tcgetattr (fd, &options);

  // Do like minicom: set 0 in speed options
  cfsetispeed (&options, 0);
  cfsetospeed (&options, 0);

  options.c_iflag = IGNBRK;

  // Enable the receiver and set local mode and 8N1
  options.c_cflag = (CLOCAL | CREAD | CS8 | HUPCL);
  // enable hardware flow control (CNEW_RTCCTS)
  // options.c_cflag |= CRTSCTS;
  // Set speed
  options.c_cflag |= baud;

  /*
     options.c_cflag &= ~PARENB;
     options.c_cflag &= ~CSTOPB;
     options.c_cflag &= ~CSIZE; // Could this be wrong!?!?!?
   */

  // set raw input
  options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
  options.c_iflag &= ~(INLCR | ICRNL | IGNCR);

  // set raw output
  options.c_oflag &= ~OPOST;
  options.c_oflag &= ~OLCUC;
  options.c_oflag &= ~ONLRET;
  options.c_oflag &= ~ONOCR;
  options.c_oflag &= ~OCRNL;

  // Set the new options for the port...
  options_cpy = options;
  tcsetattr (fd, TCSANOW, &options);
  options = options_cpy;

  // Do like minicom: set speed to 0 and back
  options.c_cflag &= ~baud;
  tcsetattr (fd, TCSANOW, &options);
  options = options_cpy;

  sleep (1);

  options.c_cflag |= baud;
  tcsetattr (fd, TCSANOW, &options);
}

/* main */
int
main (void)
{
  printf ("initppp.c\n");
  /* Wait time */
  timeout_s = 0;
  timeout_us = 500000;

  /* initial state */
  state = 0;

#if 0
  printf ("Starting initialization\n");

  /* get and activate shared memory segment */
  printf ("Getting shared memory\n");
  shmid = shmget (131073, 24, IPC_CREAT | 0600);
  if (shmid < 0) {
    printf ("error with shmget\n");
  }

  printf ("Attaching shared memory\n");
  data = shmat (shmid, NULL, 0);        //  shmat(shmid, &shmaddr, 0); //65538, 0, 0)       = 0x40016000
  shmaddr = *data;
  if (data < 0) {
    printf ("error with shmat\n");
  }

  printf ("Disabling multiplexer\n");
  multi_off ();
#endif

  printf ("Opening and initializing port\n");
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////add in 2006-07-10///////////////////////////////////////////////////
  PE1 = open_serialport ("/dev/ttyS0");
  //PE1 = open("/dev/ttyS1", O_RDWR);

#if 0
  ioctl (PE1, TCFLSH, 0x2);

  options.c_iflag = 0x1;
  options.c_oflag = 0;
  options.c_cflag = 0x1cb2;
  options.c_lflag = 0;
  options.c_line = 0;
  options.c_cc[VMIN] = 1;
  options.c_cc[VTIME] = 5;
  ioctl (PE1, SNDCTL_TMR_TIMEBASE | TCGETS, options);

  //DTR INACTIVE?
  data[8] = 0;

  options.c_iflag = 0x1;
  options.c_oflag = 0;
  options.c_cflag = 0x1cb2;
  options.c_lflag = 0;
  options.c_line = 0;
  options.c_cc[VMIN] = 1;
  options.c_cc[VTIME] = 5;
  ioctl (PE1, SNDCTL_TMR_START | TCSETS, options);

  options.c_iflag = 0x1;
  options.c_oflag = 0;
  options.c_cflag = 0x1cb2;
  options.c_lflag = 0;
  options.c_line = 0;
  options.c_cc[VMIN] = 1;
  options.c_cc[VTIME] = 5;
  ioctl (PE1, SNDCTL_TMR_TIMEBASE | TCGETS, options);

  data[8] = 1;
  ioctl (PE1, TCFLSH, 0x2);

  /* Watch PE1 to see when it has input. */
  FD_ZERO (&rfds);
  FD_SET (PE1, &rfds);
  selectn = PE1 + 1;

  /* open DTR interface */
  printf ("opening dtrri\n");
  DTRII1 = open ("/dev/dtrri", O_RDWR);

  printf ("registering SIGALRM handler\n");
  alarmaction.sa_handler = &handlealarm;
  sigaction (SIGALRM, &alarmaction, NULL);

  printf ("setting nice\n");
  nice (-15);

  printf ("opening dtrii\n");
  DTRII2 = open ("/dev/dtrri", O_RDWR);

  printf ("poking at dtrii\n");
  ioctl (DTRII2, SNDCTL_TMR_SELECT | TCSETAF, 0xbffff654);
  //DTR ACTIVE?

  options.c_iflag = 0;
  options.c_oflag = 0x2da98;
  options.c_cflag = shmaddr;
  options.c_lflag = shmaddr + 4;
  options.c_line = 4;
  options.c_cc[VMIN] = 0;
  options.c_cc[VTIME] = 0;
  ioctl (DTRII2, SNDCTL_TMR_TIMEBASE | TCGETS, options);

  ioctl (DTRII2, SNDCTL_TMR_START | TCSETS, 0);

  printf ("setting alarm\n");
  alarm (8);
  do {
    FD_ZERO (&rfds);
    FD_SET (PE1, &rfds);
    FD_ZERO (&rfds);
    FD_SET (PE1, &rfds);
    tv.tv_sec = timeout_s;
    tv.tv_usec = timeout_us;
    retval = select (selectn, &rfds, NULL, NULL, &tv);
  } while (retval == 0);

  ioctl (DTRII2, SNDCTL_TMR_SELECT | TCSETAF, 0xbffff654);

  ioctl (PE1, FIONREAD, &rsize);
  printf ("have %d bytes to read\n", rsize);
  read (PE1, &buf, rsize);
  printf ("read: [%s]\n", buf);
  do {
    FD_ZERO (&rfds);
    FD_SET (PE1, &rfds);
    tv.tv_sec = timeout_s;
    tv.tv_usec = timeout_us;
    retval = select (selectn, &rfds, NULL, NULL, &tv);
  } while (retval == 0);

  /* cancel alarm */
  printf ("cancelling alarm\n");
  alarm (0);

  do {
    FD_ZERO (&rfds);
    FD_SET (PE1, &rfds);
    tv.tv_sec = timeout_s;
    tv.tv_usec = timeout_us;
    retval = select (selectn, &rfds, NULL, NULL, &tv);
  } while (retval == 0);
  ioctl (DTRII2, SNDCTL_TMR_SELECT | TCSETAF, 0xbffff654);

  ioctl (PE1, FIONREAD, &rsize);
  printf ("have %d bytes to read\n", rsize);
  read (PE1, &buf, rsize);
  printf ("read: [%s]\n", buf);

  do {
    FD_ZERO (&rfds);
    FD_SET (PE1, &rfds);
    tv.tv_sec = timeout_s;
    tv.tv_usec = timeout_us;
    retval = select (selectn, &rfds, NULL, NULL, &tv);
  } while (retval == 0);
  ioctl (DTRII2, SNDCTL_TMR_SELECT | TCSETAF, 0xbffff654);

  ioctl (PE1, FIONREAD, &rsize);
  printf ("have %d bytes to read\n", rsize);
  read (PE1, &buf, rsize);
  printf ("read: [%s]\n", buf);

  write (PE1, "\r\n", 2);

  sleeptime.tv_sec = 0;
  sleeptime.tv_nsec = 300000000;
  nanosleep (&sleeptime, NULL);

  state = 1;

  write (PE1, "AT&F\r", 5);
  printf ("writing...");
  do {
    ioctl (PE1, TIOCOUTQ, &outq);
    printf (".");
    usleep (100);
  } while (outq > 0);
  printf ("wrote: [AT&F]\n");

  do {
    FD_ZERO (&rfds);
    FD_SET (PE1, &rfds);
    tv.tv_sec = timeout_s;
    tv.tv_usec = timeout_us;
    retval = select (selectn, &rfds, NULL, NULL, &tv);
  } while (retval == 0);
  ioctl (PE1, FIONREAD, &rsize);
  printf ("have %d bytes to read\n", rsize);
  read (PE1, &buf, rsize);
  printf ("read: [%s]\n", buf);
#endif

  writeread ("ATZ", "OK");
  /* turn multiplexing off? */
  writeread ("AT+PPMUX=0", "OK");

  /* turn echo off */
  writeread ("ATE0", "OK");

  /* reset gsm modem (set phone functionality 1=full, 1=reset) */
  //writeread("AT+CFUN=1,1", "OK");
#if 0
  /* wait for reset */
  writeread ("", "\5\20\21\36");
  /* wait a little more */
  sleep (1);
#endif

  /* +CME ERROR result codes (1=numeric) */
  writeread ("AT+CMEE=1", "OK");

  /* enable network registration and location information */
  writeread ("AT+CREG=2", "OK");

  /* extended unsolicited result code (0-255???) */
  writeread ("AT+PPURC=14207", "OK");

  /* ??? ?unknown CEC proprietary result codes? */
  //writeread("AT+PPSURC=117407487,1015551", "OK");

  /* fixed DTE rate */
  writeread ("AT+IPR=115200", "OK");

  /* ??? ?disable RTS? */
  //writeread("AT+PPRTS=0", "OK");

  /* ??? */
  writeread ("AT+PPATRIS=0", "OK");

  /* return status of ME indicators */
  //writeread("AT+CIND?", "OK");

  /* Enter online command mode by dropping DTR */
  writeread ("AT&D1", "OK");

  /* select char set */
  //writeread("AT+CSCS=\"UCS2\"", "OK");

  /* ??? ?disable ring indicator line? */
  //writeread("AT+PPRI=0", "OK");

  /* ??? */
  //writeread("AT+PPSIM=1", "OK");

  /* SIM toolkit activation (1=activate) */
  //writeread("AT+PPSTAC=1", "OK");

  /* GPRS network registation status (2=enable network registration and location URC) */
  //writeread("AT+CGREG=2", "OK");

  /* calling line identification presentation */
  //writeread("AT+CLIP=1", "OK");

  /* connected line identification presentation */
  //writeread("AT+COLP=1", "OK");

  /* enable call waiting */
  //writeread("AT+CCWA=1", "OK");

  /* enable supplementary service notifications (1=CSSI, 1=CSSU) */
  //writeread("AT+CSSN=1,1", "OK");

  /* enable new message indications ... */
  //writeread("AT+CNMI=2,5,2,1,0", "OK");

  /* enable cellular result codes (+CRING instead of RING, etc) */
  writeread ("AT+CRC=1", "OK");

  /* intermediate reporting control (for GPRS, etc) */
  writeread ("AT+CR=1", "OK");

  /* mobile equipment event report (URC for key press, display change, indicator state change, etc) 1=enable, 0=no keypad, 0=no display, 2=indicator events using +CIEV, 0=clear buffer */
  writeread ("AT+CMER=1,0,0,2,0", "OK");

  /* preferred SMS message format (0=PDU mode) (1=text mode) */
  //writeread("AT+CMGF=0", "OK");

  /* number of seconds to wait for connection completion */
  //writeread("ATS7=50", "OK");

  /* automatic answering after N rings (0=disabled) */
  //writeread("ATS0=0", "OK");

  /* alert sound mode (2=???) */
  //writeread("AT+CALM=2", "OK");

  /* request IMSI */
  writeread ("AT+CIMI", "OK");

  /* network operator selection (3=set format only, 2=numeric) */
  //writeread("AT+COPS=3,2", "OK");
  //writeread("AT+COPS=1,2,\"20820\"", "OK");



  /* ??? */
  //writeread("AT+PPEN=1", "OK");

  /* network operator selection (3=set format only, 2=numeric) */
  //writeread("AT+COPS=3,2", "OK");

  /* get operator selection mode */
  // writeread("AT+COPS?", "OK");

  /* get indicator control mode */
  //writeread("AT+CIND?", "OK");

  /* request SMS memory status (x=used, y=available) */
  //writeread("AT+PPSMSM?", "OK");

  /* unchanged SMS status (reads an SMS without changing state) */
  //writeread("AT+PPUSMS=1", "OK");

  /* unchanged SMS status (reads an SMS without changing state) */
  //writeread("AT+PPUSMS=2", "OK");

  /* unchanged SMS status (reads an SMS without changing state) */
  //writeread("AT+PPUSMS=3", "OK");
///////////////////////////add in 2006-07-11//////////////////////////////////////////////
  //close(PE1);
  return 0;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////add in 2006-07-10/////////////////////////
int
open_serialport (char *dev)
{
  int fd;

  fd = serial_opendevice (dev, FALSE, FALSE);
  if (fd == -1) {
    printf ("open modem failed\n");
    return -1;
  }

  serial_setdtrrts (fd, 1, 1);
  return fd;
}

static Int8 g_inbuf[ATCOMMAND_BUFFER_SIZE];
static UInt16 g_total = 0;

#define ERR_NONE 0
#define ERR_HAPPEN -1
#define ERR_NO_DATA -2

int
read_loop (char *resp)
{
  fd_set readfds;
  Int32 res = 0;
  Int32 n_read = 0;
  int result;
  struct timeval timeout;

  timeout.tv_sec = 0;
  timeout.tv_usec = 100000;     /* 100ms */

  FD_ZERO (&readfds);
  FD_SET (PE1, &readfds);
  res = select (PE1 + 1, &readfds, NULL, NULL, &timeout);
  if (res > 0) {
    printf ("have data to read!!!!!!!!!!!\n");
    // read all buffer out
    while (1) {
      memset (g_inbuf, 0, ATCOMMAND_BUFFER_SIZE);
      n_read = serial_read (PE1, g_inbuf, ATCOMMAND_BUFFER_SIZE);
      printf ("g_inbuf is %s, n_read is %d\n", g_inbuf, n_read);
      if (n_read <= 0) {
        break;
      }
      g_total += n_read;
      if (strstr (g_inbuf, resp))
        return ERR_NONE;
      else if (strstr (g_inbuf, "ERROR"))
        return ERR_HAPPEN;
    }
  }
  printf ("no more data to read for the time being\n");
  return ERR_NO_DATA;
}


int
GetResponse_Handler (char *resp)
{
  struct timeval now, next, timeout;
  int result;

  timeout.tv_sec = SERIAL_TIMEOUT;
  timeout.tv_usec = 0;
  gettimeofday (&now, NULL);
  timeradd (&now, &timeout, &next);

  /* retry read command response every 100ms within a timeout */
  do {
    result = read_loop (resp);
    gettimeofday (&now, NULL);
    //if within 100ms we found an error, then report error
    //otherwise we wait for the response to complete
    if ((result == ERR_NONE) || (result == ERR_HAPPEN)) {
      return result;
    }
  } while (timercmp (&next, &now, >));

  return result;
}


static int
xwrite (Int8 * d, UInt32 len)
{
  UInt32 res;

  printf ("date to write is %d\n", len);

  while (len) {
    res = serial_write (PE1, d, len, -1);
    if (res == -1) {
      if (errno != EAGAIN) {
        perror ("gnokii I/O error ");
        return -1;
      }
    } else {
      //      g_print ("really in? +++\n");
      d += res;
      len -= res;
    }
  }

  printf ("date remained is %d\n", len);
  return 0;
}


init_general (char *command, char *resp)
{
  int result, reallen;
  UInt8 retry;
  UInt8 tmp;
  char realcommand[ATCOMMAND_BUFFER_SIZE];


  sprintf (realcommand, "%s\r", command);
  printf ("realcommand is %s\n", command);
  reallen = strlen (realcommand);
  for (retry = 0; retry < SERIAL_RETRY_NUMBER; retry++) {
    memset (g_inbuf, 0, ATCOMMAND_BUFFER_SIZE);
    g_total = 0;

    result = xwrite (realcommand, reallen);
    if (result == ERR_NONE) {
      result = GetResponse_Handler (resp);
      if (result == ERR_NONE) {
        break;
      }
      //else sleep 0.5s
      else {
        printf ("resend command\n");
        sleep (1);
      }
    }
  }
  return result;
}

int
writeread (char *writestr, char *resp)
{
  init_general (writestr, resp);
}
