/*	connect - Application of connect/disconnect to GPRS
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <sys/soundcard.h>
#include <sys/types.h>
#include <asm/ioctls.h>
#include <time.h>

/* for select */
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

/* for shmget, shmat */
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>

/* for stat */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

//JD
#include <glib.h>
#include "dbus-if.h"
#include "shared.h"


/* for signal */
//#include <signal.h>

/* Serial Port Handle */
int PE1;

struct termios options;
struct timespec sleeptime;

int rsize;
char buf[8193];

fd_set rfds;
struct timeval tv;
int retval;
int selectn;
int timeout_s;
int timeout_us;

struct stat statbuf;

int shmid;
int shmaddr;
char *data;

int outq;

//struct sigaction alarmaction;

//int state=0;
char *laststr;
char *lastresp;

//JD
static DBusConnection *bus = NULL;

//JDD

/* send AT command to GSM modem and wait for response */
int
writeread (char *writestr, char *resp)
{
  char mystr[8193] = "";
  int len = strlen (writestr);

  laststr = writestr;
  lastresp = resp;
  int numread = 0;
  char *bufptr = mystr;

  alarm (30);
  strcpy (mystr, writestr);
  strcat (mystr, "\r");
  write (PE1, mystr, len + 1);
  printf ("writing [%s]...", writestr);
  do {
    ioctl (PE1, TIOCOUTQ, &outq);
    printf (".");
    usleep (100);
  } while (outq > 0);
  printf ("done.\nreading...");
  strncpy (mystr, "", 8192);
  do {                          // until found resp
    printf ("waiting on %s for %s\n", resp, writestr);
    do {                        // until select 
      FD_ZERO (&rfds);
      FD_SET (PE1, &rfds);
      tv.tv_sec = timeout_s;
      tv.tv_usec = timeout_us;
      retval = select (selectn, &rfds, NULL, NULL, &tv);
      printf (".");
    } while (retval == 0);
    ioctl (PE1, FIONREAD, &rsize);      //[46]                = 0
    numread = read (PE1, bufptr, rsize);        //"\r\n+PPURC:256,1\r\n\r\n+CIEV: 0,4\r\n\r\n"..., 4096);// = 46
    printf ("done.\nread %d bytes into mystr=[%s].\n", numread, mystr);
    bufptr = mystr + strlen (mystr);    /* read next on top of null terminating char */
  } while (!strstr (mystr, resp) && !strstr (mystr, "ERROR"));
  //alarm(0);
  if (strstr (mystr, "ERROR")) {
    /* received error.  give up for now */
    printf ("got error, bailing out!\n");
    return (1);
  } else {
    printf ("have %s, done with writeread.\n", resp);
    return (0);
  }
}

void
setAdvancedOptions (int fd, speed_t baud)
{
  struct termios options;
  struct termios options_cpy;

  fcntl (fd, F_SETFL, 0);

  // get the parameters
  tcgetattr (fd, &options);

  // Do like minicom: set 0 in speed options
  cfsetispeed (&options, 0);
  cfsetospeed (&options, 0);

  options.c_iflag = IGNBRK;

  // Enable the receiver and set local mode and 8N1
  options.c_cflag = (CLOCAL | CREAD | CS8 | HUPCL);

  // Set speed
  options.c_cflag |= baud;

  // set raw input
  options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
  options.c_iflag &= ~(INLCR | ICRNL | IGNCR);

  // set raw output
  options.c_oflag &= ~OPOST;
  options.c_oflag &= ~OLCUC;
  options.c_oflag &= ~ONLRET;
  options.c_oflag &= ~ONOCR;
  options.c_oflag &= ~OCRNL;

  // Set the new options for the port...
  options_cpy = options;
  tcsetattr (fd, TCSANOW, &options);
  options = options_cpy;

  // Do like minicom: set speed to 0 and back
  options.c_cflag &= ~baud;
  tcsetattr (fd, TCSANOW, &options);
  options = options_cpy;

  sleep (1);

  options.c_cflag |= baud;
  tcsetattr (fd, TCSANOW, &options);
}




static void
ps_mc_cb (DBusMessage * message)
{
  g_print ("!!!!!!!!!11111111111111111event from Phoneserver received!\n");

  DBusError error;
  DBusMessage *reply;

  dbus_error_init (&error);

  guint32 res;


  if (!dbus_message_get_no_reply (message))     //sender expects a reply
  {
    reply = dbus_message_new_method_return (message);
    dbus_connection_send (bus, reply, NULL);
    dbus_message_unref (reply);
  }

  if (dbus_message_get_args (message, &error,
                             DBUS_TYPE_UINT32, &res, DBUS_TYPE_INVALID)) {
    g_print ("res=%d, ", res);
    if (res == 0) {
      char *gprspath = "/mnt/doc/vroot/program";        //getenv("GPRS_PATH");

      printf ("\n\n\n#################################%s", gprspath);
      const char *cmd_connect =
        "pppd /dev/pts/1 user _dummy connect $GPRS_PATH/connect_mul call gprsmux &";
      system (cmd_connect);
    }
  } else {
    g_print ("EVENT Message received, but error getting message: %s\n",
             error.message);
    dbus_error_free (&error);
  }
}


int
mul_service_init ()
{
  DBusError error;

  bus = dbus_init ();
  g_print ("in int\n");
  if (!bus)
    return -1;

  dbus_error_init (&error);
  dbus_bus_request_name (bus, PHONE_MUL_SERVICE, 0, &error);
  if (dbus_error_is_set (&error)) {
    g_warning ("Failed to acquire service:%s", error.message);
    dbus_error_free (&error);
    return 1;
  }

  g_print ("the unique name for this connection: %s\n",
           dbus_bus_get_unique_name (bus));
  dbus_add_msg_listener_mc (PHONE_MUL_INTERFACE, PHONE_MUL_DELIVER_EVENT,
                            ps_mc_cb);
}


static gboolean
mul_send_evt_to_ps (DBusConnection * bus, char *pmethod)
{
  DBusMessage *message;
  DBusError error;

  dbus_error_init (&error);

  message = dbus_message_new_method_call (PHONE_SERVER_SERVICE,
                                          PHONE_SERVER_MUL_OBJECT,
                                          PHONE_SERVER_MUL_INTERFACE,
                                          PHONE_SERVER_MUL_METHOD_SEND);


  if (message == NULL) {
    printf ("Failed to contruct message\n");
    dbus_error_free (&error);
    return FALSE;
  }

  /* Append the event to the signal */
  //when method=dial, the lid not cid is added
  //in other ocasions, cid is added
  dbus_message_append_args (message,
                            DBUS_TYPE_STRING, &pmethod, DBUS_TYPE_INVALID);

  DBusMessage *reply = NULL;

  dbus_error_init (&error);
  reply =
    dbus_connection_send_with_reply_and_block (bus, message, 60000, &error);
  if (dbus_error_is_set (&error)) {
    g_print ("Could not get create a new call!error: %s\n", error.message);
    dbus_error_free (&error);

    return FALSE;
  }
  guint32 res;

  if (dbus_message_get_args (reply, &error,
                             DBUS_TYPE_UINT32, &res, DBUS_TYPE_INVALID)) {
    g_print ("successfully getting the res code = %d!\n", res);
  } else {
    g_print ("error getting cid with error: %s\n", error.message);
    dbus_error_free (&error);
    dbus_message_unref (message);
    dbus_message_unref (reply);
    return FALSE;
  }
  dbus_message_unref (message);
  dbus_message_unref (reply);

  g_print ("Message sendout!\n");
  /* Return TRUE to tell the event loop we want to be called again */
  return TRUE;
}


int
mul_connect (void)
{
  char *pmethod = "gconnect";

  if (mul_send_evt_to_ps (bus, pmethod))
    return TRUE;
  return TRUE;
}



int
mul_disconnect (void)
{
  char *pmethod = "gdisconnect";

  if (mul_send_evt_to_ps (bus, pmethod))
    return TRUE;
  return TRUE;
}


/* main */
int
mul_dev_init (void)
{
  /* Wait time */
  timeout_s = 0;
  timeout_us = 500000;

  /* initial state */
  //state = 0;

  /* get and activate shared memory segment */
  printf ("Getting shared memory\n");
  shmid = shmget (131073, 24, IPC_CREAT | 0600);
  if (shmid < 0) {
    printf ("error with shmget\n");
  }

  printf ("Attaching shared memory\n");
  data = shmat (shmid, NULL, 0);        //  shmat(shmid, &shmaddr, 0); //65538, 0, 0)       = 0x40016000
  shmaddr = *data;
  if (data < 0) {
    printf ("error with shmat\n");
  }

  PE1 = open_serialport ("/dev/pts/0");
  ioctl (PE1, TCFLSH, 0x2);

  options.c_iflag = 0x1;
  options.c_oflag = 0;
  options.c_cflag = 0x1cb2;
  options.c_lflag = 0;
  options.c_line = 0;
  options.c_cc[VMIN] = 1;
  options.c_cc[VTIME] = 5;
  ioctl (PE1, SNDCTL_TMR_TIMEBASE | TCGETS, options);

  //DTR INACTIVE?
  data[8] = 0;

  options.c_iflag = 0x1;
  options.c_oflag = 0;
  options.c_cflag = 0x1cb2;
  options.c_lflag = 0;
  options.c_line = 0;
  options.c_cc[VMIN] = 1;
  options.c_cc[VTIME] = 5;
  ioctl (PE1, SNDCTL_TMR_START | TCSETS, options);
  options.c_iflag = 0x1;
  options.c_oflag = 0;
  options.c_cflag = 0x1cb2;
  options.c_lflag = 0;
  options.c_line = 0;
  options.c_cc[VMIN] = 1;
  options.c_cc[VTIME] = 5;
  ioctl (PE1, SNDCTL_TMR_TIMEBASE | TCGETS, options);

  data[8] = 1;
  ioctl (PE1, TCFLSH, 0x2);

  /* Watch PE1 to see when it has input. */
  FD_ZERO (&rfds);
  FD_SET (PE1, &rfds);
  selectn = PE1 + 1;



  printf ("\n\n\n###################start debug\n");


  printf ("INIT is  finished\n");

  return 0;
}

int
open_serialport (char *dev)
{
  int fd;

  fd = open (dev, O_RDWR | O_NOCTTY | O_NDELAY);
  if (fd != -1) {
    //setAdvancedOptions(fd, B115200);           
  }
  return fd;
}

