/*	connect - Application of connect/disconnect to GPRS
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include "dbus-if.h"
#include "shared.h"
//#include "vocerror.h"
//#include "voctype.h"


  static GList *listeners = NULL;
  static DBusConnection *bus = NULL;

  typedef struct
  {
    gchar *interface;
    gchar *method;
    void (*callback) ();
  } t_listener;

  static DBusHandlerResult msg_filter (DBusConnection * connection,
                                       DBusMessage * message, void *user_data)
  {

    /* User data is the event loop we are running in */
    GMainLoop *loop = (GMainLoop *) user_data;
    GList *iter;
    for (iter = listeners; iter; iter = iter->next) {
      t_listener *li = (t_listener *) iter->data;

      if (dbus_message_is_signal (message, li->interface, li->method))
      {
        li->callback (message);
        return DBUS_HANDLER_RESULT_HANDLED;
      } else
      {
        if (dbus_message_is_method_call (message, li->interface, li->method)) {
          //li->callback(connection, message, NULL); 
          li->callback (message);
          return DBUS_HANDLER_RESULT_HANDLED;
        }
      }

    }

    /* A signal from the bus saying we are about to be disconnected */
    if (dbus_message_is_signal
        (message, DBUS_INTERFACE_LOCAL, "Disconnected")) {
      /* Tell the main loop to quit */
      g_main_loop_quit (loop);

      /* We have handled this message, don't pass it on */
      return DBUS_HANDLER_RESULT_HANDLED;
    }

    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
  }

  void
    dbus_add_msg_listener (const gchar * interface, const gchar * method,
                           void (*callback) ())
  {
//fix me, using hash table may be better.

    t_listener *new_listener = g_malloc0 (sizeof (t_listener));
    gchar *filterstr =
      g_strdup_printf ("type='signal',interface='%s'", interface);

    new_listener->interface = g_strdup (interface);
    new_listener->method = g_strdup (method);
    new_listener->callback = callback;

    listeners = g_list_append (listeners, (gpointer) new_listener);

    dbus_bus_add_match (bus, filterstr, NULL);
    g_free (filterstr);
  }


  void
    dbus_add_msg_listener_mc (const gchar * interface, const gchar * method,
                              void (*callback) ())
  {
//fix me, using hash table may be better.

    t_listener *new_listener = g_malloc0 (sizeof (t_listener));
    gchar *filterstr =
      g_strdup_printf ("type='method_call',interface='%s'", interface);

    new_listener->interface = g_strdup (interface);
    new_listener->method = g_strdup (method);
    new_listener->callback = callback;

    listeners = g_list_append (listeners, (gpointer) new_listener);

    dbus_bus_add_match (bus, filterstr, NULL);

    g_free (filterstr);

  }


  DBusConnection *dbus_init ()
  {
    DBusError error;
    GMainLoop *mainloop;

    mainloop = g_main_loop_new (NULL, TRUE);
    dbus_error_init (&error);

    bus = dbus_bus_get (DBUS_BUS_SESSION, &error);
    if (bus == NULL) {
      g_printerr ("Failed to get session bus connection %s", error.message);
      dbus_error_free (&error);
      return NULL;
    }

    dbus_connection_setup_with_g_main (bus, NULL);

    /* we do not request a namespace to support multiple listeners to these signals */
    /* listening to messages from all objects as no path is specified */


    dbus_connection_add_filter (bus, msg_filter, mainloop, NULL);
//dbus_add_msg_listener_mc ( PHONE_VOICE_INTERFACE, VOC_DBUS_METHOD_SEND_EVENT, cb_dial2);


    //g_main_loop_unref(mainloop);

    return bus;
  }

/*
vi:ts=2:nowrap:ai:expandtab
*/
